@@ -1,5 +1,5 @@
 /*
- * $Id: axis.h,v 1.166.2.1 2017/07/29 16:30:14 sfeam Exp $
+ * $Id: axis.h,v 1.166.2.4 2017/09/29 19:02:17 sfeam Exp $
  *
  */
 
@@ -106,13 +106,6 @@ typedef enum en_ticseries_type {
     TIC_DAY      		/* print out day of week */
 } t_ticseries_type;
 
-typedef enum {
-    DT_NORMAL=0,		/* default; treat values as pure numeric */
-    DT_TIMEDATE,		/* old datatype */
-    DT_DMS,			/* degrees minutes seconds */
-    DT_UNINITIALIZED
-} td_type;
-
 /* Defines one ticmark for TIC_USER style.
  * If label==NULL, the value is printed with the usual format string.
  * else, it is used as the format string (note that it may be a constant
@@ -360,6 +353,9 @@ extern int grid_layer;
 /* Whether to draw the axis tic labels and tic marks in front of everything else */
 extern TBOOLEAN grid_tics_in_front;
 
+/* Whether to draw vertical grid lines in 3D */
+extern TBOOLEAN grid_vertical_lines;
+
 /* Whether or not to draw a separate polar axis in polar mode */
 extern TBOOLEAN raxis;
 
@@ -648,7 +644,7 @@ double quantize_normal_tics __PROTO((double, int));
 void setup_tics __PROTO((struct axis *, int));
 void gen_tics __PROTO((struct axis *, tic_callback));
 void axis_output_tics __PROTO((AXIS_INDEX, int *, AXIS_INDEX, tic_callback));
-void axis_set_scale_and_range __PROTO((struct axis *axis, unsigned int lower, unsigned int upper));
+void axis_set_scale_and_range __PROTO((struct axis *axis, int lower, int upper));
 void axis_draw_2d_zeroaxis __PROTO((AXIS_INDEX, AXIS_INDEX));
 TBOOLEAN some_grid_selected __PROTO((void));
 void add_tic_user __PROTO((struct axis *, char *, double, int));
@@ -673,6 +669,8 @@ void clone_linked_axes __PROTO((AXIS *axis1, AXIS *axis2));
 AXIS *get_shadow_axis __PROTO((AXIS *axis));
 void extend_primary_ticrange __PROTO((AXIS *axis));
 void update_primary_axis_range __PROTO((struct axis *secondary));
+void update_secondary_axis_range __PROTO((struct axis *primary));
+void reconcile_linked_axes __PROTO((AXIS *primary, AXIS *secondary));
 
 int map_x __PROTO((double value));
 int map_y __PROTO((double value));
@@ -700,4 +698,9 @@ double eval_link_function __PROTO((AXIS *, double));
      ticlevel < MAX_TICLEVEL ? ticscale[ticlevel] : \
      0)
 
+/* convenience macro to make sure min < max */
+#define reorder_if_necessary( min, max ) \
+do { if (max < min) { double temp = min; min = max; max = temp; } \
+} while (0)
+
 #endif /* GNUPLOT_AXIS_H */
