@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: command.c,v 1.354.2.3 2017/08/30 18:37:06 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - command.c */
 
 /*[
@@ -806,7 +802,7 @@ array_command()
 TBOOLEAN
 is_array_assignment()
 {
-    udvt_entry *udv = add_udv(c_token);
+    udvt_entry *udv;
     struct value newvalue;
     int index;
     TBOOLEAN looks_OK = FALSE;
@@ -836,6 +832,7 @@ is_array_assignment()
     if (!looks_OK)
 	return FALSE;
 
+    udv = add_udv(c_token);
     if (udv->udv_value.type != ARRAY)
 	int_error(c_token, "Not a known array");
 
@@ -2665,13 +2662,28 @@ static float splot_map_surface_rot_x;
 static float splot_map_surface_rot_z;
 static float splot_map_surface_scale;
 
+/* utility routine for splot_map_activate/deactivate */
+static void
+flip_y_axis()
+{
+    struct axis *yaxis = &axis_array[FIRST_Y_AXIS];
+    double temp = yaxis->min;
+    yaxis->min = yaxis->max;
+    yaxis->max = temp;
+    if (yaxis->linked_to_primary) {
+	yaxis = yaxis->linked_to_primary;
+	temp = yaxis->min;
+	yaxis->min = yaxis->max;
+	yaxis->max = temp;
+    }
+}
+
 /* This routine is called at the beginning of 'splot'. It sets up some splot
  * parameters needed to present the 'set view map'.
  */
 void
 splot_map_activate()
 {
-    double temp;
     if (splot_map_active)
 	return;
     splot_map_active = 1;
@@ -2685,9 +2697,7 @@ splot_map_activate()
     /* version 4 had constant value surface_scale = 1.3 */
     surface_scale = 1.425 * mapview_scale;
     /* The Y axis runs backwards from a normal 2D plot */
-    temp = axis_array[FIRST_Y_AXIS].min;
-    axis_array[FIRST_Y_AXIS].min = axis_array[FIRST_Y_AXIS].max;
-    axis_array[FIRST_Y_AXIS].max = temp;
+    flip_y_axis();
 }
 
 
@@ -2697,7 +2707,6 @@ splot_map_activate()
 void
 splot_map_deactivate()
 {
-    double temp;
     if (!splot_map_active)
 	return;
     splot_map_active = 0;
@@ -2706,9 +2715,7 @@ splot_map_deactivate()
     surface_rot_z = splot_map_surface_rot_z;
     surface_scale = splot_map_surface_scale;
     /* The Y axis runs backwards from a normal 2D plot */
-    temp = axis_array[FIRST_Y_AXIS].min;
-    axis_array[FIRST_Y_AXIS].min = axis_array[FIRST_Y_AXIS].max;
-    axis_array[FIRST_Y_AXIS].max = temp;
+    flip_y_axis();
 }
 
 
@@ -3135,11 +3142,11 @@ is_history_command(const char *line)
     const char str[] = "hi$story";
 
     /* skip leading whitespace */
-    while (isblank((unsigned char) line[start]))
+    while (isspace((unsigned char) line[start]))
 	start++;
 
     /* find end of "token" */
-    while ((line[start + length] != NUL) && !isblank((unsigned char) line[start + length]))
+    while ((line[start + length] != NUL) && !isspace((unsigned char) line[start + length]))
 	length++;
 
     for (i = 0; i < length + after; i++) {
@@ -3637,10 +3644,6 @@ do_system_func(const char *cmd, char **output)
 
 }
 
-#if defined(_WIN32) && !defined(WEXITSTATUS)
-#define WEXITSTATUS(error) (error)
-#endif
-
 static int
 report_error(int ierr)
 {
