@@ -1,5 +1,5 @@
 /*
- * $Id: datafile.h,v 1.49.2.1 2016/03/19 04:06:21 sfeam Exp $
+ * $Id: datafile.h,v 1.52 2016/08/19 16:51:58 sfeam Exp $
  */
 
 /* GNUPLOT - datafile.h */
@@ -104,6 +104,7 @@ extern struct udft_entry ydata_func; /* deprecated "thru" function */
 
 /* Returned to caller by df_readline() */
 extern char *df_tokens[];
+extern struct value df_strings[];	/* used only by TABLESTYLE */
 
 /* number of columns in first row of data return to user in STATS_columns */
 extern int df_last_col;
@@ -131,11 +132,15 @@ extern TBOOLEAN df_nofpe_trap;
 extern TBOOLEAN evaluate_inside_using;
 extern TBOOLEAN df_warn_on_missing_columnheader;
 
+/* Used by plot title columnhead, stats name columnhead */
+extern char *df_key_title;
+
 /* Prototypes of functions exported by datafile.c */
 
 int df_open __PROTO((const char *, int, struct curve_points *));
 int df_readline __PROTO((double [], int));
 void df_close __PROTO((void));
+void df_init __PROTO((void));
 char * df_fgets __PROTO((FILE *));
 void df_showdata __PROTO((void));
 int df_2dbinary __PROTO((struct curve_points *));
