@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: internal.c,v 1.96.2.3 2017/08/30 18:37:07 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - internal.c */
 
 /*[
@@ -1495,11 +1491,18 @@ f_sprintf(union argument *arg)
 
 	/* Check for %%; print as literal and don't consume a parameter */
 	if (!strncmp(next_start,"%%",2)) {
+	    remaining++;	/* Don't consume a parameter value */
 	    next_start++;
+	    next_length = strcspn(next_start+1,"%") + 1;
+	    prev_pos = outpos - buffer;
+	    while (prev_pos + next_length >= bufsize) {
+		bufsize *= 2;
+		buffer = gp_realloc(buffer, bufsize, "f_sprintf");
+		outpos = buffer + prev_pos;
+	    }
 	    do {
 		*outpos++ = *next_start++;
 	    } while(*next_start && *next_start != '%');
-	    remaining++;
 	    continue;
 	}
 
@@ -1515,7 +1518,6 @@ f_sprintf(union argument *arg)
 	if ( spec_type != STRING && next_param->type == STRING )
 	    int_error(NO_CARET,"f_sprintf: attempt to print string value with numeric format");
 
-#ifdef HAVE_SNPRINTF
 	/* Use the format to print next arg */
 	switch(spec_type) {
 	case INTGR:
@@ -1533,22 +1535,6 @@ f_sprintf(union argument *arg)
 	default:
 	    int_error(NO_CARET,"internal error: invalid spec_type");
 	}
-#else
-	/* FIXME - this is bad; we should dummy up an snprintf equivalent */
-	switch(spec_type) {
-	case INTGR:
-	    sprintf(outpos, next_start, (int)real(next_param));
-	    break;
-	case CMPLX:
-	    sprintf(outpos, next_start, real(next_param));
-	    break;
-	case STRING:
-	    sprintf(outpos, next_start, next_param->v.string_val);
-	    break;
-	default:
-	    int_error(NO_CARET,"internal error: invalid spec_type");
-	}
-#endif
 
 	next_start[next_length] = tempchar;
 	next_start += next_length;
@@ -1665,7 +1651,7 @@ f_strftime(union argument *arg)
     /* Get time_str */
     length = gstrftime(buffer, buflen, fmtstr, real(&val));
     if (length == 0 || length >= buflen)
-	int_error(NO_CARET, "Resulting string is too long");
+	int_error(NO_CARET, "String produced by time format is too long");
 
     /* Remove trailing space */
     assert(buffer[length-1] == ' ');
@@ -1701,15 +1687,15 @@ f_strptime(union argument *arg)
 
 
     /* string -> time_tm  plus extra fractional second */
-    gstrptime(val.v.string_val, fmt.v.string_val, &time_tm, &usec);
-
-    /* time_tm -> result */
-    result = gtimegm(&time_tm);
+    if (gstrptime(val.v.string_val, fmt.v.string_val, &time_tm, &usec, &result)
+	    == DT_TIMEDATE) {
+	/* time_tm -> result */
+	result = gtimegm(&time_tm);
+	/* Add back any extra fractional second */
+	result += usec;
+    }
     FPRINTF((stderr," strptime result = %g seconds \n", result));
 
-    /* Add back any extra fractional second */
-    result += usec;
-
     gpfree_string(&val);
     gpfree_string(&fmt);
     push(Gcomplex(&val, result, 0.0));
