@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: plot3d.c,v 1.263.2.5 2017/08/23 01:00:05 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - plot3d.c */
 
 /*[
@@ -94,6 +90,9 @@ static void thin_plate_splines_setup __PROTO(( struct iso_curve *old_iso_crvs, d
 static double qnorm __PROTO(( double dist_x, double dist_y, int q ));
 static double pythag __PROTO(( double dx, double dy ));
 
+/* helper function to detect empty data sets */
+static void count_3dpoints __PROTO((struct surface_points *plot, int *nt, int *ni, int *nu));
+
 /* helper functions for parsing */
 static void load_contour_label_options __PROTO((struct text_label *contour_label));
 
@@ -548,11 +547,9 @@ grid_nongrid_data(struct surface_points *this_plot)
     struct iso_curve *icrv, *oicrv, *oicrvs;
     
     /* these are only needed for thin_plate_splines */
-    double *xx, *yy, *zz, *b;
+    double *xx = NULL, *yy = NULL, *zz = NULL, *b = NULL;
     int numpoints = 0;
 
-    xx = NULL; /* save to call free() on NULL if xx has never been used */
-    
     /* Compute XY bounding box on the original data. */
     /* FIXME HBB 20010424: Does this make any sense? Shouldn't we just
      * use whatever the x and y ranges have been found to be, and
@@ -1558,6 +1555,9 @@ eval_3dplots()
 	    else
 		load_linetype(&this_plot->lp_properties, line_num+1);
 
+	    /* FIXME:  We may have cleared these previously? Anyhow it doesn't hurt. */
+	    this_plot->opt_out_of_hidden3d = FALSE;
+
 	    /* pm 25.11.2001 allow any order of options */
 	    while (!END_OF_COMMAND || !checked_once) {
 		int save_token = c_token;
@@ -1820,9 +1820,6 @@ eval_3dplots()
 		&&  (this_plot->lp_properties.p_size == PTSZ_VARIABLE))
 		    this_plot->lp_properties.p_size = 1;
 	    }
-	    if (this_plot->plot_style == LINES) {
-		this_plot->opt_out_of_hidden3d = FALSE;
-	    }
 
 	    if (crnt_param == 0
 		&& this_plot->plot_style != PM3DSURFACE
@@ -1883,6 +1880,7 @@ eval_3dplots()
 		    this_plot->token = this_token;
 
 		    df_return = get_3ddata(this_plot);
+
 		    /* for second pass */
 		    this_plot->token = c_token;
 		    this_plot->iteration = plot_iterator ? plot_iterator->iteration : 0;
@@ -1890,6 +1888,17 @@ eval_3dplots()
 		    if (this_plot->num_iso_read == 0)
 			this_plot->plot_type = NODATA;
 
+		    /* Sep 2017 - Check for all points bad or out of range  */
+		    /* (normally harmless but must not cause infinite loop) */
+		    if (forever_iteration(plot_iterator)) {
+			int ntotal, ninrange, nundefined;
+			count_3dpoints(this_plot, &ntotal, &ninrange, &nundefined);
+			if (ninrange == 0) {
+			    this_plot->plot_type = NODATA;
+			    goto SKIPPED_EMPTY_FILE;
+			}
+		    }
+
 		    if (this_plot != first_dataset)
 			/* copy (explicit) "with pm3d at ..." option from the first dataset in the file */
 			strcpy(this_plot->pm3d_where, first_dataset->pm3d_where);
@@ -2205,6 +2214,7 @@ eval_3dplots()
     if (nonlinear(&axis_array[FIRST_X_AXIS])) {
 	/* Transfer observed data or function ranges back to primary axes */
 	update_primary_axis_range(&axis_array[FIRST_X_AXIS]);
+	extend_primary_ticrange(&axis_array[FIRST_X_AXIS]);
 	axis_check_empty_nonlinear(&axis_array[FIRST_X_AXIS]);
     } else {
 	axis_checked_extend_empty_range(FIRST_X_AXIS, "All points x value undefined");
@@ -2213,6 +2223,7 @@ eval_3dplots()
     if (nonlinear(&axis_array[FIRST_Y_AXIS])) {
 	/* Transfer observed data or function ranges back to primary axes */
 	update_primary_axis_range(&axis_array[FIRST_Y_AXIS]);
+	extend_primary_ticrange(&axis_array[FIRST_Y_AXIS]);
 	axis_check_empty_nonlinear(&axis_array[FIRST_Y_AXIS]);
     } else {
 	axis_checked_extend_empty_range(FIRST_Y_AXIS, "All points y value undefined");
@@ -2470,3 +2481,26 @@ static void load_contour_label_options (struct text_label *contour_label)
     lp->flags |= LP_SHOW_POINTS;
     lp_parse(lp, LP_ADHOC, TRUE);
 }
+
+/* Count the number of data points but do nothing with them */
+static void
+count_3dpoints(struct surface_points *plot, int *ntotal, int *ninrange, int *nundefined)
+{
+    int i;
+    struct iso_curve *icrvs = plot->iso_crvs;
+    *ntotal = *ninrange = *nundefined = 0;
+
+    while (icrvs) {
+	struct coordinate GPHUGE *point;
+	for (i = 0; i < icrvs->p_count; i++) {
+	    point = &(icrvs->points[i]);
+	    (*ntotal)++;
+	    if (point->type == INRANGE)
+		(*ninrange)++;
+	    if (point->type == UNDEFINED)
+		(*nundefined)++;
+	}
+	icrvs = icrvs->next;
+    }
+}
+
